#!/bin/sh
UPDLOG=/usr/kingfisher/var/update.txt
CM_PIDLIST=/tmp/.cmprocess.pid
TARFILE=$1
USRTMPDIR=/usr/tmp/update
SAVEFILE=save1.tgz
mkdir -p /usr/kingfisher/var

# for KF protocol upgrade, move file to RAM disk, same as for SSH.
# updater will find it there when it is restarted below
if [ "`dirname $TARFILE`" != "/tmp" ]; then
    mv $TARFILE /tmp
    mv `dirname $TARFILE`/.update /tmp
    TARFILE=/tmp/`basename $TARFILE`
fi

# run updater from RAM disk so we can unmount /usr
if [ ! -e /tmp/update ]; then
    echo "121 Starting updater" > $UPDLOG
    echo "121 Starting updater"
    cp /usr/kingfisher/sbin/update /tmp/update
    tar -xzOf $TARFILE ./usr/kingfisher/sbin/watchdog >/tmp/watchdog
    chmod +x /tmp/watchdog
    killall update
    /tmp/update $1
    exit 0
fi

# check that existing firmware is not too old
if [ -e /usr/kingfisher/etc/firmware -a `cat /usr/kingfisher/etc/firmware | sed s/M//` -lt 2874 ]; then
    echo "950 Upgrade via intermediate version is required" > $UPDLOG
    exit 9;
fi

# check for at least 200k free space in /tmp (need to store updater binary)
kbfree=`df | awk '/\/tmp/ { print $4; }'`
[ $kbfree ] || kbfree=`df | awk '/\/dev\/ramdisk/ { print $4; }'`
[ $kbfree ] || kbfree=`df | awk '/\/dev\/root/ { print $4; }'`
[ $kbfree ] || kbfree=10000
if [ $kbfree -lt 200 ]; then
    echo "951 Insufficient temporary storage - try again" > $UPDLOG
    exit 9;
fi

logger -t preinst "Firmware upgrade started" 2>/dev/null


# stop application processes - see shutdown.sh
echo "122 Stopping processes" > $UPDLOG

# - port/protocol processes
if [ -e $CM_PIDLIST ]; then
    kflist=`cat $CM_PIDLIST`
    kill -SIGKILL $kflist
else
    kflist=""
    for f in `find /usr/kingfisher/bin -type f`; do 
        kflist="$kflist `basename $f`"
    done
    killall -q -SIGKILL $kflist eth mcport modem optx radio serial
fi

# - other processes
killall -q -SIGKILL kingfisher evflush ntpclient monitor
killall -q -SIGKILL ETCP IsaVM ConfigurationManager

# Workaround for old checker possibly timing out during upgrade:
#  use watchdog utility to kick hw watchdog instead of checker
# Also need to start watchdog process to ensure hardware WDT is active
#  on reboot to avoid possible flash lockup after reset
# Run from RAM disk so we can unmount /usr
echo 0 >/dev/watchdog0
killall -q -SIGKILL checker watchdog watchdog.sh
/tmp/watchdog &

if tar -tzf $TARFILE ./usr/tmp/update/usr.img; then

    # clear event log
    rm /usr/kingfisher/var/eventlog.bin

    # before wiping /usr, save required files (if they exist)
    echo "123 Saving settings" > $UPDLOG
    rm -f usr/kingfisher/var/debug*
    mkdir -p $USRTMPDIR
    tar -C/ -czf $USRTMPDIR/$SAVEFILE \
    usr/kingfisher/var/*.log \
    usr/kingfisher/var/*.log.* \
    usr/kingfisher/var/syslog00.txt \
    usr/kingfisher/var/rstflag \
    usr/kingfisher/data/data.1.1 \
    usr/kingfisher/etc/passwd \
    usr/kingfisher/etc/shadow \
    usr/kingfisher/etc/nologic \
    usr/kingfisher/etc/dropbear* \
    usr/kingfisher/var/usr.tgz \
    /usr/config/*.txt \
    /usr/v2000/app/I* \
    >/dev/null 2>&1

    # also clear Isagraf logic, just in case we need the space
    # (will be restored in postinstall)
    cp /usr/config/default/v2000/app/* /usr/v2000/app
fi

# updater will now extract files from upgrade package

exit 0

